<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use Schema;

use DateTime;
use App\Models\Setting;
use App\Models\Language;
use App\Models\Company;
use App\Models\Testimony;
use App\Models\Partnership;
use App\Models\CoreMember;
use App\Models\Job;
use App\Models\Faq;
use App\Models\Article;
use App\Models\ArticleDescription;
use App\Models\Category;
use App\Models\ContentManagement;

class JoinSeaContentsController extends Controller
{
    public function __construct()
    {
        $this->page_variables = [
                                    'page_title'   =>   Language::trans('Cities Page'),
                                    'return_url' => class_basename($this).'@getIndex',
                                    'edit_link' => class_basename($this).'@getEdit' ,
                                    'view_link' => class_basename($this).'@getView' ,
                                    'delete_link' => class_basename($this).'@getDelete',
                                    'new_file_link' => class_basename($this).'@getNew' 
                                ];

        //$this->middleware('acl', ['only'=>['getIndex','getNew','getEdit','getView','getDelete']]);
        $this->middleware('cors');
    }

   public function getCompany()
   {
        $multi_text_vars = ['email','tel'];
        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $t_languages = [['id'=>'0','name'=>'Enlisgh','cover_photo_path'=>'https://cdn-icons-png.flaticon.com/512/2983/2983798.png']];
        $photo_keys = ['logo_photo_path','favicon_photo_path'];
        $translate_keys = ['social_medias'];
        $company = Company::find(Setting::project_company_id);

        foreach($translate_keys as $t_key)
        {
            if(isset($company[$t_key]))
            {
                $new_sm = array();
                $temp = $company[$t_key] != null ?  json_decode($company[$t_key],true) : '';
                if(is_array($temp))
                {
                        foreach($temp as $sm_key => $sm_val)
                        {
                            if($sm_val == null){continue;}
                            $temp_sm = ['item'=>  $social_media_mapper[$sm_key],'url'=>$sm_val];
                            array_push($new_sm,$temp_sm);

                        }
                       
                        $company[$t_key] = $new_sm;
                }
                
            }  

        }

        foreach($photo_keys as $p_key)
        {
            if(isset($company[$p_key]))
            {
                //$row['cover_photo_path'] = 'http://localhost/joinsea/public/img/testimony_cover_photo/testimony_24_08_2023_08_00_43.jpeg';
                $company[$p_key] = asset($company[$p_key]);
                
            } 
        }

        foreach($multi_text_vars as $m_key)
        {
            if(isset($company[$m_key]))
            {
                $temp = explode(',', $company[$m_key]);
                $company[$m_key] = $temp;
            }
        }


        $company['languages'] = $t_languages;
        $company['address'] = $company->get_address();
        
        //dd($company);

        return json_encode($company);

   }

   public function getPageUrl()
   {
        $page = 'page url setting';
        $language = 'en';
        $return = array();
        $translate_keys = ['title','description','content','more_url'];
        $listing = ContentManagement::getByPageName($page);
        

        foreach($listing as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';
                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                $return[$row['type']] = ['title' => $row['title'] , 'url' => $row['more_url']];
            }   

        }

        return json_encode($return);

   } 
   
   public function getArticle($id)
   {
        $language = 'en';
        $return = array();
        $child_result_keys = ['cover_photo_path','title','share_title','share_description','hashtags','content','slugs','meta'];
        $parent_result_keys = ['id','category_id','publish_at'];


        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $photo_keys = ['logo_photo_path','favicon_photo_path','cover_photo_path'];
        $translate_keys = ['title','description','content'];
        $company = Company::find(Setting::project_company_id);
        $categories = Category::getByKey('type','article');
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }

        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
        $article_per_page = 5;
        $model = Article::find($id);         
        
        $c_row = array();
        $temp = array();
        $children_list = ArticleDescription::getByKey('article_id',$model['id']);
        $model->convertDateToString('publish_at');
        /*$date = new DateTime($model['publish_at']);
        $model['publish_at'] = $date->format("F j, Y");*/


        foreach($children_list as $c_data)
        {
             if($c_data['language'] == $language)
             {

                $c_row = $c_data;
                foreach($photo_keys as $p_key)
                {
                    if(isset($c_row[$p_key]))
                    {
                        //$model['cover_photo_path'] = 'http://localhost/joinsea/public/img/testimony_cover_photo/testimony_24_08_2023_08_00_43.jpeg';
                        $c_row[$p_key] = asset($c_data[$p_key]);
                        
                    } 
                }

             }
         }

        foreach($child_result_keys as $c_key)
        {
            $return[$c_key] = isset($c_row[$c_key]) ? $c_row[$c_key] : '';
        }

        foreach($parent_result_keys as $p_key)
        {
            $return[$p_key] = isset($model[$p_key]) ? $model[$p_key] : '';
        }
 

        $return['categories'] = $categories;

        return json_encode($return);

   }

   public function getLatestArticles()
   {
        $language = 'en';
        $return = array();
        $child_result_keys = ['cover_photo_path','title','share_title','share_description','hashtags','content','slugs','meta'];
        $parent_result_keys = ['id','category_id','publish_at'];

        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $photo_keys = ['logo_photo_path','favicon_photo_path','cover_photo_path'];
        $translate_keys = ['title','description','content'];
        $company = Company::find(Setting::project_company_id);
        $categories = Category::getByKey('type','article');
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }

        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
        $article_per_page = 5;
        $listing = Article::getByPage($page,$article_per_page);

        $model = new Article();
        
        $counter = 1;
        foreach($listing as $row)
        {
            if($counter >  4){continue;}
            $counter ++;
            $c_row = array();
            $temp = array();
            $children_list = ArticleDescription::getByKey('article_id',$row['id']);
            $row->convertDateToString('publish_at');
        
            foreach($children_list as $c_data)
            {
                 if($c_data['language'] == $language)
                 {

                    $c_row = $c_data;
                    foreach($photo_keys as $p_key)
                    {
                        if(isset($c_row[$p_key]))
                        {
                            //$row['cover_photo_path'] = 'http://localhost/joinsea/public/img/testimony_cover_photo/testimony_24_08_2023_08_00_43.jpeg';
                            $c_row[$p_key] = asset($c_data[$p_key]);
                            
                        } 
                    }

                 }
             }

            foreach($child_result_keys as $c_key)
            {
                $temp[$c_key] = isset($c_row[$c_key]) ? $c_row[$c_key] : '';
            }

            foreach($parent_result_keys as $p_key)
            {
                $temp[$p_key] = isset($row[$p_key]) ? $row[$p_key] : '';
            }
            

            array_push($return  , $temp);
           
        }
    

        return json_encode($return);

   }

   public function getArticles()
   {
        $language = 'en';
        $return = array();
        $child_result_keys = ['cover_photo_path','title','share_title','share_description','hashtags','content','slugs','meta'];
        $parent_result_keys = ['id','category_id','publish_at'];

        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $photo_keys = ['logo_photo_path','favicon_photo_path','cover_photo_path'];
        $translate_keys = ['title','description','content'];
        $company = Company::find(Setting::project_company_id);
        $categories = Category::getByKey('type','article');
        //dd($categories);
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }

        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
        $article_per_page = 5;
        $listing = Article::getByPage($page,$article_per_page);

        $model = new Article();
        
       
        foreach($listing as $row)
        {
            $c_row = array();
            $temp = array();
            $children_list = ArticleDescription::getByKey('article_id',$row['id']);
            $row->convertDateToString('publish_at');
        
            foreach($children_list as $c_data)
            {
                 if($c_data['language'] == $language)
                 {

                    $c_row = $c_data;
                    foreach($photo_keys as $p_key)
                    {
                        if(isset($c_row[$p_key]))
                        {
                            //$row['cover_photo_path'] = 'http://localhost/joinsea/public/img/testimony_cover_photo/testimony_24_08_2023_08_00_43.jpeg';
                            $c_row[$p_key] = asset($c_data[$p_key]);
                            
                        } 
                    }

                 }
             }

            foreach($child_result_keys as $c_key)
            {
                $temp[$c_key] = isset($c_row[$c_key]) ? $c_row[$c_key] : '';
            }

            foreach($parent_result_keys as $p_key)
            {
                $temp[$p_key] = isset($row[$p_key]) ? $row[$p_key] : '';
            }
            
            $return['articles'][$row['category_id']] = isset($return['articles'][$row['category_id']])  ? $return['articles'][$row['category_id']] : array();
            array_push($return['articles'][$row['category_id']]  , $temp);
           
        }
    
        $return['categories'] = $categories;
        //dd($categories);
        //dd($return);
        return json_encode($return);

   }

   public function getJob($id)
   {
        $language = 'en';
        $return = array();
        $copy_keys = ['id','slot_needed'];
        $parent_result_keys = ['id','category_id','publish_at'];

        $model = Job::find($id);        
        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $photo_keys = ['logo_photo_path','favicon_photo_path','cover_photo_path'];
        $translate_keys = ['title','content'];
        $company = Company::find(Setting::project_company_id);
        $categories = Category::getByKey('type','job');
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }

        


        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
    
       
        $temp = array();

        foreach($translate_keys as $c_key)
        {
            if(!isset($model[$c_key])){continue;}
            $temp = $model[$c_key] != null ?  json_decode($model[$c_key],true) : '';

            $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
   
            $return[$c_key] = $temp_c;
             //dd($row);
        }  

        foreach($copy_keys as $c_key)
        {
            $return[$c_key] = isset($model[$c_key]) ? $model[$c_key] : '';
        }

        $model->convertDateToString('publish_at');
        $return['categories'] = $categories;

        return json_encode($return);

   }

   public function getJobs()
   {
        $language = 'en';
        $return = array();
        $post_contents = array();
        $job_natures = Setting::getJobNatures();
        $job_types = Setting::getJobTypes();
        $model_keys = ['slot_needed','location','working_period'];
        $translate_keys = ['title','description','content'];
        $content_keys = ['title','share_title','share_description','content'];
        $categories = Category::getByKey('type','job');
        //dd($categories);
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }

        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
        $item_per_page = 5;
        $listing = Job::getByPage($page,$item_per_page);

        $contents = array();
        foreach($listing as $model)
        {
            foreach($content_keys as $var)
            {
                $$var = isset($model[$var]) ? json_decode($model[$var] , true) : false;
                if($$var != false)
                {
                    foreach($$var as $l_key => $l_val)
                    {
                        if($l_key == $language){
                            $post_contents[$model['id']][$var] = $l_val;
                        }
                        
                    }
                }
            }
        }
        

        foreach($listing as $row)
        {
            $temp = array();

            $temp['job_nature'] = $job_natures[$row['job_nature']];
            $temp['job_type'] = $job_types[$row['job_type']];

            $salary_range = explode(';',$row['salary_range']);
            $from = is_int($salary_range[0]) ? $salary_range[0] : '';
            $to = is_int($salary_range[1]) ? $salary_range[1] : '';

            if($to == '' && $from == '')
            {
                $temp['salary_range']  = Language::trans('To be discuss.');
            }else if($to == $from)
            {
                $temp['salary_range']  = $to;
            }else{
                $temp['salary_range'] =  Language::trans('From').' '.$from.' - '.$to;
            }
            
            $temp['experience']  = $row['experience'].' '.Language::trans('years');


            $post_content = isset($post_contents[$row['id']]) ? $post_contents[$row['id']] : array();
            foreach($content_keys as $c_key)
            {
                 $temp[$c_key] = isset($post_content[$c_key]) ? $post_content[$c_key] : '';
            }

            foreach($model_keys as $c_key)
            {
                 $temp[$c_key] = isset($model[$c_key]) ? ( $model[$c_key] == null ? $model[$c_key] : '' ) : '';
            }

            $return['jobs'][$row['category_id']] = isset($return[$row['category_id']])  ? $return[$row['category_id']] : array();

            $temp['id'] = isset($row['id']) ? $row['id'] : 999;
            array_push($return['jobs'][$row['category_id']]  , $temp);
           
        }
    
        $return['categories'] = $categories;
        //dd($return);
        return json_encode($return);

   }


   public function getFaqs()
   {
        $language = 'english';
        $t_language = 'en';
        $return = array();
        $post_contents = array();
        $translate_keys = ['title','description','content'];
        $content_keys = ['title','share_title','share_description','content'];
        $categories = Category::getByKey('type','faq');
        //dd($categories);
        foreach($categories as $row)
        {

            foreach($translate_keys as $c_key)
            {
                if(!isset($row[$c_key])){continue;}
                $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                $temp_c = is_array($temp) == true ? (isset($temp[$t_language]) ? $temp[$t_language] : '') : '';
                $row[$c_key] = $temp_c;
                 //dd($row);
            }   

        }
        // Validate and sanitize the 'page' parameter from the query string
        $page = filter_input(INPUT_GET, 'page', FILTER_VALIDATE_INT);

        // Set a default value of 1 if 'page' is not provided or is not a valid integer
        if ($page === false || $page === null) {
            $page = 1;
        }
        $item_per_page = 5;
        $listing = Faq::all();
        //dd($listing);
        $contents = array();
        foreach($listing as $model)
        {
            foreach($content_keys as $var)
            {
                $$var = isset($model[$var]) ? json_decode($model[$var] , true) : false;
                if($$var != false)
                {
                    foreach($$var as $l_key => $l_val)
                    {
                        //echo $l_key . ': ' . $l_val.'<br>';
                        if($l_key == $language){
                            $post_contents[$model['id']][$var] = $l_val;
                        }else if($l_key == $t_language){
                            $post_contents[$model['id']][$var] = $l_val;
                        }
                        
                    }
                }
            }
        }
        
        //dd($post_contents);
        foreach($listing as $row)
        {

            //echo $row['id'].'<br>';
            $post_content = isset($post_contents[$row['id']]) ? $post_contents[$row['id']] : array();
            foreach($content_keys as $c_key)
            {
                 $temp[$c_key] = isset($post_content[$c_key]) ? $post_content[$c_key] : '';
            }

            //echo json_encode($temp).'<br>';

            $return['faqs'][$row['category_id']] = isset($return['faqs'][$row['category_id']])  ? $return['faqs'][$row['category_id']] : array();
            //echo $row['category_id'].'<br>';
            $temp['id'] = isset($row['id']) ? $row['id'] : 999;
            array_push($return['faqs'][$row['category_id']]  , $temp);
           
        }
    
        $return['categories'] = $categories;
        //dd($return);
        return json_encode($return);

   }


   public function getHomePageData()
   {
        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $language = 'english';
        $return = array();
        $testimonies = array();
        $keys = ['testimony','partnership','core_member','faq','category'];
        $translate_keys = ['content'];
        $convert_keys = ['content','title'];

        foreach($keys as $key)
        {
            $arr_name =  Setting::pluralize($key);
            $$arr_name = array();

            $className = Setting::getModelClass($key);
            $temp_list = $className::getLatest(10);
            $counter  = 1;
            foreach($temp_list as $row)
            {

                foreach($convert_keys as $c_key)
                {
                    $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                    $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                    $row[$c_key] = $temp_c;
                     //dd($row);
                }   


                if(isset($row['social_medias']))
                {
                    $new_sm = array();
                    $temp = $row['social_medias'] != null ?  json_decode($row['social_medias'],true) : '';
                    if(is_array($temp))
                    {
                            foreach($temp as $sm_key => $sm_val)
                            {
                                if($sm_val == null){continue;}
                                $temp_sm = ['item'=>$social_media_mapper[$sm_key],'url'=>$sm_val];
                                array_push($new_sm,$temp_sm);

                            }
                           
                            $row['social_medias'] = $new_sm;
                    }
                    
                }  

                if(isset($row['cover_photo_path']))
                {
                    //$row['cover_photo_path'] = 'http://localhost/joinsea/public/img/testimony_cover_photo/testimony_24_08_2023_08_00_43.jpeg';
                    $row['cover_photo_path'] = asset($row['cover_photo_path']);
                    
                } 

                if($key == 'core_member')
                {
                    if($counter <= 2)
                    {
                        $row['layer'] = ''; 
                        $row['col_class'] = 'col-md-6';
                    }else{
                        $row['layer'] = '-two';
                        $row['col_class'] = 'col-lg-4 col-md-6 col-sm-10';
                    }
                  
                }

                $counter ++;
                array_push($$arr_name,$row);
            }

            

             $return[$arr_name] = $$arr_name;
            
        }

        $faqs = array();
        foreach($return['faqs'] as $row)
        {
            $faqs[$row['category_id']] = isset($faqs[$row['category_id']]) ? $faqs[$row['category_id']] : array();
            array_push($faqs[$row['category_id']],$row);
        }

        $return['faqs'] = $faqs ;
        //dd($row);
        return json_encode($return);

   }


   public function getContentByPage($request_type)
   {

        $home_items =  ['statistic', 'feature','switch','statistic','timeline','tool' , 'tool_title','switch_title','feature_title','statistic_title','timeline_title','home_title', ];
        $about_us_items =  ['statistic', 'introduction','home_title' , 'statistic_title','introduction_title'];
        $contact_us_items =  [];
        $how_it_works_items =  ['step','step_title'];
        $footer_items =  ['first_row','first_row_title','second_row','second_row_title','third_row','third_row_title','community_title','community_box'];

        $page_type_mappers = [
            'home' => $home_items,
            'about us' => $about_us_items,
            'contact_us' => $contact_us_items ,
            'how it works' => $how_it_works_items,
            'footer' => $footer_items
        ];

        $language = 'en';
        $return = array();
        $child_result_keys = ['cover_photo_path','title','share_title','share_description','hashtags','content','slugs','meta'];
        $parent_result_keys = ['id','category_id','publish_at'];


        $social_media_mapper = ['twitter'=>'fab fa-twitter','facebook'=>'fab fa-facebook-f','linkedin'=>'fab fa-linkedin-in'];
        $photo_keys = ['logo_photo_path','favicon_photo_path','cover_photo_path'];
        $translate_keys = ['title','description','content','content_2','hightlighted_text','more_text','more_url'];

        $company = Company::find(Setting::project_company_id);

        $contents = array();
        $request_items = isset($page_type_mappers[$request_type])  ? $page_type_mappers[$request_type] : false;
        if($request_items == false){ return false;}


        $return = array();
        foreach($request_items as $type)
        {
            $temp_listing = array();
            $listing = array();
            $listing = ContentManagement::getByPageAndType($request_type,$type);
            $index = 1;
            foreach($listing as $row)
            {
                $row['new_hightlighted_text'] = array();
                if($row['hightlighted_text'] !== null)
                {
                    $hts =   json_decode($row['hightlighted_text'],true) ;
                    $temp_hts = isset($hts[$language]) ? $hts[$language] : false;
                    if($temp_hts != false)
                    {
                        $e_hts = explode(',',$temp_hts);
                        $temp_title = $row['title'];
                        foreach($e_hts as $ht)
                        {

                           $row['title'] = str_replace($ht, '<span>'.$ht.'</span>', $temp_title);
                        }

                         $row['new_hightlighted_text'] = $e_hts;
                    }
                }
                //process highlighted text
                foreach($translate_keys as $c_key)
                {
                    if(!isset($row[$c_key])){continue;}
                    $temp = $row[$c_key] != null ?  json_decode($row[$c_key],true) : '';

                    $temp_c = is_array($temp) == true ? (isset($temp[$language]) ? $temp[$language] : '') : '';
                    $row[$c_key] = $temp_c;
                } 

                $cpp = $row['cover_photo_path'];
                $temp_cpp = json_decode($row['cover_photo_path'],true);
                if(is_array($temp_cpp))
                {
                    $cpp = isseT($temp_cpp[$language]) ? $temp_cpp[$language] : $cpp;
                }

                $row['cover_photo_path'] = asset($cpp);
                $row['index'] = $index;
                $index++;
                
            }

            $return[$type] = $listing;
        }
        //dd($return);
        return json_encode($return);

   }

}
