<?php

namespace App\Models;

use DB;
use Auth;
use Schema;
use Validator;

use Illuminate\Database\Eloquent\Builder;

class Category extends ExtendModel
{
    protected $table = 'categories';
    public $timestamps = true;
    protected $listing_except_columns = ['leaf_group_id','parent_id','child_id','ky_group_id','code','title','content','remark','created_at','created_by','updated_at','updated_by'];

    protected $guarded = [];

    /*
    |--------------------------------------------------------------------------
    | Here to manage of relationships
    |--------------------------------------------------------------------------
    |
    */

    public function creator()
    {
        return $this->belongsTo('App\User', 'created_by');
    }

    public function updator()
    {
        return $this->belongsTo('App\User', 'updated_by');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('owned_by', function (Builder $builder) {
            $builder->where('ky_group_id', '=', Company::get_ky_group_id());
        });
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of data's
    |--------------------------------------------------------------------------
    |
    */

    public static function combobox($category_id=null)
    {
        return static::ofAvailable('status',true)
                                /*->where('id','=',$category_id)*/
                                ->pluck('name','id')
                                ->prepend(Language::trans('Please select Category...'), '');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of index listing displayed
    |--------------------------------------------------------------------------
    |
    */

    public function table_cols()
    {
        $except = $this->listing_except_columns;

        return array_diff(Schema::getColumnListing($this->table), $except);
    }

    public function listing_header()
    {
        return array_diff($this->table_cols(), $this->listing_except_columns);
    }

    public function scopeListing($query) 
    {
        return $query->select(array_diff($this->table_cols(), $this->listing_except_columns));
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of validation & save form
    |--------------------------------------------------------------------------
    |
    */

    public static function getLatest($offset=false)
    {
        $return = static::select('*')
                        ->orderBy('sort_order','asc');
        if($offset != false)
        {
            $return = $return->skip(0)->take($offset);
        }

        return $return->get();
    }

    public function validate_form($input)
    {
        $rules = [
                    'state_id'  =>  'required',
                    'code'      =>  'required|unique:categories,code,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    'name'      =>  'required|unique:categories,name,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    ];

        if ($this->id) {
            $rules['code'] = 'required|unique:categories,code,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
            $rules['name'] = 'required|unique:categories,name,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
        }

        $validator = Validator::make($input, $rules);

        if ($validator->fails()) {
            return $validator;
        }
        return false;
    }

    const state_keys = ['status'];
    const custom_keys =  ['icon_1'];
    public function save_form($input)
    {
        //dd($input);
        //dd( static::getArrayKeys());
        $content_keys = ['title','content'];
        DB::beginTransaction();
        try {
            foreach ($input as $key => $value) {
                if (!in_array($key , static::custom_keys) && !in_array($key , static::getStateKeys()) && !in_array($key , static::getGlobalExcludeKeys()) && !in_array($key , static::getArrayKeys())) {
                    //echo $key.'<br>';
                    if(is_array($value)){
                        //dd($key);
                        continue;
                    }
                    $this->$key = (string) $value;
                    if($key == 'category_type')
                    {
                        $this->type = $value;
                    }

                }elseif(in_array($key, static::getStateKeys()))
                {
                    $this->$key = $value ;

                }elseif(in_array($key, static::custom_keys))
                {
                    $this->icon = $value ;

                }else if(in_array($key, static::getArrayKeys()))
                {
                    foreach($content_keys  as $ck)
                    {
                        $$ck = array();
                    }
                    
                    $description = array();
                    if($key == 'contents')
                    {
                        foreach($value as $language_key => $val)
                        {
                            foreach($val as $v_key => $v_val)
                            {
                                $$v_key[$language_key] = $v_val;
                            }
                            
                        }
                    }

                    foreach($content_keys  as $ck)
                    {
                        $this->$ck = json_encode($$ck);
                    }


                }
            }
            if (!$this->id) {
                $this->created_by       =   Auth::id() ? Auth::id():0;
                $this->updated_by       =   0;
                $this->leaf_group_id    =   Company::get_group_id();
                $this->updated_by       =   Auth::id() ? Auth::id():0;
            } else {
                $this->updated_by       =   Auth::id() ? Auth::id():0;
            }

            foreach(Company::getSaveModelDefaultKeys() as $d_key => $d_value){
                 $this->$d_key = $d_value;
             }
 
            //d($this);
            $this->save();
        } catch (Exception $e) {
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }
}
