<?php

namespace App\Models;

use DB;
use Auth;
use Schema;
use Validator;
use Request;
use Image;
use App\Models\EventDescription;

use Illuminate\Database\Eloquent\Builder;

class Event extends ExtendModel
{
    protected $table = 'events';
    public $timestamps = true;
    protected $listing_except_columns = ['hashtags','is_suspend','country_id','google_map_location','leaf_group_id','ky_group_id','sub_category_id','creator_facebook_url','user_facebook_app_id','user_facebook_page_id','youtube_links','created_by','updated_by','cancelled_by','created_at','updated_at'];







    protected $guarded = [];

    /*
    |--------------------------------------------------------------------------
    | Here to manage of relationships
    |--------------------------------------------------------------------------
    |
    */

    public function creator()
    {
        return $this->belongsTo('App\Models\User', 'created_by');
    }

    public function updator()
    {
        return $this->belongsTo('App\Models\User', 'updated_by');
    }

    public function event()
    {
        return $this->belongsTo('App\Models\Event', 'event_id');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('owned_by', function (Builder $builder) {
            $builder->where('leaf_group_id', '=', Company::get_group_id());
        });
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of data's
    |--------------------------------------------------------------------------
    |
    */

    public static function combobox($state_id=null)
    {
        return static::ofAvailable('status',true)
                                ->where('state_id','=',$state_id)
                                ->pluck('name','id')
                                ->prepend(Language::trans('Please select Event...'), '');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of index listing displayed
    |--------------------------------------------------------------------------
    |
    */

    public function table_cols()
    {
        $except = $this->listing_except_columns;

        return array_diff(Schema::getColumnListing($this->table), $except);
    }

    public function listing_header()
    {
        return array_diff($this->table_cols(), $this->listing_except_columns);
    }

    public function scopeListing($query) 
    {
        return $query->select(array_diff($this->table_cols(), $this->listing_except_columns));
    }

    /*
    |--------------------------------------------------------------------------
    | Setter and getter
    |--------------------------------------------------------------------------
    |
    */
    public static function getRecentEvent($date = false)
    {
        $date = $date != false ? $date : date('Y-m-d', strtotime('now'));
        $return = static::where('held_at','<=', $date)->get();


        return $return;
    }


    /*
    |--------------------------------------------------------------------------
    | Here to manage of validation & save form
    |--------------------------------------------------------------------------
    |
    */

    public function validate_form($input)
    {
        $rules = [
                    'state_id'  =>  'required',
                    'code'      =>  'required|unique:events,code,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    'name'      =>  'required|unique:events,name,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    ];

        if ($this->id) {
            $rules['code'] = 'required|unique:events,code,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
            $rules['name'] = 'required|unique:events,name,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
        }

        $validator = Validator::make($input, $rules);

        if ($validator->fails()) {
            return $validator;
        }
        return false;
    }

    const excluded_keys = [];
    const child_key = ['contents'];
    const array_keys = ['category_ids', 'gallery_photos'];
      
    public function save_form($input)
    {
        //dd( static::getGlobalExcludeKeys());
        $request_type = isset($input['request_type']) ? $input['request_type'] : false;
        $model = isset($input['id']) ?   ( $input['id'] != null ? static::find($input['id']) : new static() ) : new static();
        DB::beginTransaction();
        try {

            $default_keys = Company::getSaveModelDefaultKeys();
            foreach ($input as $key => $value) 
            {
                if (!in_array($key , static::getPhotoKeys()) && !in_array($key , static::getStateKeys()) && !in_array($key , static::excluded_keys) && !in_array($key , static::getGlobalExcludeKeys()) && !in_array($key , static::child_key) && !in_array($key , static::array_keys)) {
                   //echo 'K :'.$key.'<br>';
                   if(is_array($value))
                   {
                        //echo 'Array Key :'.$key.'<br>';
                        continue;
                   }
                   if($key == 'daterange')
                   {
                        $this->held_at = (string) $value;
                   }else{
                        $model[$key] = (string) $value;
                   }
                    

                }else if(in_array($key, static::getPhotoKeys()))
                {
                    //dd($input);
                    if (Request::hasFile($key)) {
                        if (Request::file($key)) {
                            $file = Request::file($key);
                                //$new_file = $file;

                                $destination    =   SETTING::getPhotoPath('event');
                                $i = 1;
                                $img = Image::make($file->getRealPath());
                                $extension      =   'jpeg'; //$file->getClientOriginalExtension();
                                $folder = public_path().'/'.$destination;
                                //$file_name = $file->getClientOriginalName().'-'.date('d-m-Y-H-i-s').'-'.$i.'.'.$extension;
                                $file_name = $request_type.date('d_m_Y_H_i_s').'.'.$extension;
                                //dd($file_name);
                                $file->move($folder,$file_name);
                                //dd($destination);
                                //$photo_path  = Image::make($new_file)->resize(200, 200)->save();
                                //save into directory and update path to database

                                if($model[$key]){
                                    @unlink(public_path($model[$key]));
                                }

                                $model[$key] = $destination.$file_name;

                        }
                    }

                }else if(in_array($key, static::array_keys))
                {
                    if($key == 'gallery_photos')
                    {
                        
                        $g_photos = array();
                        if (Request::hasFile($key)) {

                            if (Request::file($key)) {
                                $files = Request::file($key);
                                //dd($file);
                                $p_counter = 0;

                                foreach($files as $file)
                                {
                                    //dd($file);
                                    //$new_file = $file;

                                    $destination    =   SETTING::event_photo_path;
                                    $i = 1;
                                    $img = Image::make($file->getRealPath());
                                    $extension      =   'jpeg'; //$file->getClientOriginalExtension();
                                    $folder = public_path().'/'.$destination;
                                    $file_name = $file->getClientOriginalName().'-'.date('d-m-Y-H-i-s').'-'.$i.'.'.$extension;
                                    $file->move($folder,$file_name);
                                    //dd($destination);
                                    //$photo_path  = Image::make($new_file)->resize(200, 200)->save();
                                    //save into directory and update path to database

                                    if($model[$key]){
                                        @unlink(public_path($model[$key]));
                                    }

                                    //$photo_path = $destination.$input[$key]->getClientOriginalName().'-'.date('d-m-Y-H-i-s').'-'.$i.'.'.$extension;
                                    //$img->save($photo_path);
                                    $temp_p = ['path' => $destination.$file_name , 'sort_order' => $p_counter];
                                    //$temp_p = ['path' => $folder.$file_name , 'sort_order' => $p_counter];
                                    array_push($g_photos,$temp_p);
                                    $p_counter ++;
                                }
                                
                            }
                         }


                         $model[$key]  = json_encode($g_photos);
                    }else{
                        $model[$key] = json_encode($value);
                    }
                  

                }else if(in_array($key, static::child_key))
                {
                     if ($model['id']) {
                        //EventDescription::delete('DELETE FROM event_descriptio WHERE event_id = ?', [$model['id']]);
                        $m_listing = EventDescription::find($model['id']);

                        if ($m_listing) {
                            // Call the delete() method on the instance
                            $m_listing->delete();
                        } else {
                            // Handle the case when the model with the given ID is not found
                            // You can log an error, show a message, etc.
                        }
                     }

                     foreach($value as $language_id => $content_data){
                        $ev_model = new EventDescription();
                        foreach ($content_data as $c_key => $c_value) {
                            $ev_model[$c_key] = $c_value;
                        }

                        foreach($default_keys as $d_key => $d_value){
                            $ev_model[$d_key] = $d_value;
                        }

                        $ev_model->save();
                     }
                    
                }
                //End special key check
            }

                
            
            foreach($default_keys as $d_key => $d_value){
                 $model[$d_key] = $d_value;
             }
            
            if (!$model['id']) {
                $model['created_by']       =   Auth::id() ? Auth::id():0;
                $model['updated_by']       =   0;
                $model['leaf_group_id']    =   Company::get_group_id();
                $model['updated_by']       =   Auth::id() ? Auth::id():0;
            } else {
                $model['updated_by']       =   Auth::id() ? Auth::id():0;
            }
           
            foreach($model->attributes as $key => $value){
                echo 'Attribute :'.$key.'<br>';
                if(strpos($key,'_id') !== false)
                {
                    if($key == 'id'){

                      continue;
                    }
                    if($value == '' || $value == null || $value == 'null'){

                        $model[$key] = 0;
                    }
                }
            }
            //dd($this);
            $this->save();
        } catch (Exception $e) {
            dd($e);
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }
}
