<?php

namespace App\Models;

use DB;
use Auth;
use Schema;
use Validator;
use Request;
use Image;
use Symfony\Component\HttpFoundation\File\UploadedFile;

use Illuminate\Database\Eloquent\Builder;

class GeneralContent extends ExtendModel
{
    protected $table = 'general_contents';
    public $timestamps = true;
    protected $listing_except_columns = ['hashtags','is_suspend','country_id','google_map_location','leaf_group_id','ky_group_id','sub_category_id','creator_facebook_url','user_facebook_app_id','user_facebook_page_id','youtube_links','created_by','updated_by','cancelled_by','created_at','updated_at'];

    protected $guarded = [];

    /*
    |--------------------------------------------------------------------------
    | Here to manage of relationships
    |--------------------------------------------------------------------------
    |
    */

    public function creator()
    {
        return $this->belongsTo('App\Models\User', 'created_by');
    }

    public function updator()
    {
        return $this->belongsTo('App\Models\User', 'updated_by');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('owned_by', function (Builder $builder) {
            //$builder->where('leaf_group_id', '=', Company::get_group_id());
            $builder->where('status', '=', true);
        });
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of data's
    |--------------------------------------------------------------------------
    |
    */

    public static function combobox($state_id=null)
    {
        return static::ofAvailable('status',true)
                                ->where('state_id','=',$state_id)
                                ->pluck('name','id')
                                ->prepend(Language::trans('Please select GeneralContent...'), '');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of index listing displayed
    |--------------------------------------------------------------------------
    |
    */

    public function table_cols()
    {
        $except = $this->listing_except_columns;

        return array_diff(Schema::getColumnListing($this->table), $except);
    }

    public function listing_header()
    {
        return array_diff($this->table_cols(), $this->listing_except_columns);
    }

    public function scopeListing($query) 
    {
        return $query->select(array_diff($this->table_cols(), $this->listing_except_columns));
    }

    /*
    |--------------------------------------------------------------------------
    | Setter and getter
    |--------------------------------------------------------------------------
    |
    */
    public static function getRecentGeneralContent($date = false)
    {
        $date = $date != false ? $date : date('Y-m-d', strtotime('now'));
        $return = static::where('held_at','<=', $date)->get();


        return $return;
    }

    public static function getLatest($offset=false)
    {
        $return = static::select('*')
                        ->orderBy('sort_order','asc');
        if($offset != false)
        {
            $return = $return->skip(0)->take($offset);
        }

        return $return->get();
    }


    /*
    |--------------------------------------------------------------------------
    | Here to manage of validation & save form
    |--------------------------------------------------------------------------
    |
    */

    public function validate_form($input)
    {
        $rules = [
                    'state_id'  =>  'required',
                    'code'      =>  'required|unique:general_contents,code,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    'name'      =>  'required|unique:general_contents,name,NULL,id,leaf_group_id,'.Company::get_group_id(),
                    ];

        if ($this->id) {
            $rules['code'] = 'required|unique:general_contents,code,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
            $rules['name'] = 'required|unique:general_contents,name,'.$this->id.',id,leaf_group_id,'.Company::get_group_id();
        }

        $validator = Validator::make($input, $rules);

        if ($validator->fails()) {
            return $validator;
        }
        return false;
    }

    const excluded_keys = ['id','is_with_attachment'];
    const child_key = ['contents'];
    const array_keys = ['category_ids', 'gallery_photos','content'];
      
    public function save_form($input)
    {
        $cover_photo_paths = isset($input['cover_photo_path']) ? $input['cover_photo_path'] : false;
        $inputs = isset($input[$input['request_type']]) ? $input[$input['request_type']] : false;
        
        if($inputs == 'false')
        {
            return false;
        }

        DB::beginTransaction();
        try {
            foreach($inputs as $i_index => $input)
            {
                $is_with_attachment = isset($input['is_with_attachment']) ? $input['is_with_attachment'] : false;
                $default_keys = Company::getSaveModelDefaultKeys();
                        //echo 'input :'.json_encode($input).'<br>';
                        $model = isset($input['id']) ?   ( $input['id'] != null ? GeneralContent::find($input['id']) : new GeneralContent() ) : new GeneralContent();
                        //echo 'Model :'.json_encode($model).'<br>';

                        foreach ($input as $key => $value) 
                        {
                 
                            if (!in_array($key , static::getStateKeys()) && !in_array($key , static::excluded_keys) && !in_array($key , static::getGlobalExcludeKeys()) && !in_array($key , static::getPhotoKeys()) && !in_array($key , static::array_keys)) {
               
                               if(is_array($value))
                               {
                                    continue;
                               }
                               
                                $model[$key] = (string) $value;
                               
                                
                            }else if($key == 'cover_photo_path'){
             
                                if (Request::hasFile($key)) {
                              
                                    if (Request::file($key)) {
                                        $file = Request::file($key);
                                        
                                            //dd($file);
                                            //$new_file = $file;

                                            $destination    =   SETTING::getPhotoPath('GeneralContent_cover_photo');
                                            $i = 1;
                                            $img = Image::make($file->getRealPath());
                                            $extension      =   'jpeg'; //$file->getClientOriginalExtension();
                                            $folder = public_path().'/'.$destination;
                                            //$file_name = $file->getClientOriginalName().'-'.date('d-m-Y-H-i-s').'-'.$i.'.'.$extension;
                                            $file_name = 'GeneralContent_'.date('d_m_Y_H_i_s').'.'.$extension;
                                            //dd($file_name);
                                            $file->move($folder,$file_name);
                                            //dd($destination);
                                            //$photo_path  = Image::make($new_file)->resize(200, 200)->save();
                                            //save into directory and update path to database

                                            if($model[$key]){
                                                @unlink(public_path($model[$key]));
                                            }
                                            //dd($model);
                                            $model[$key] = $destination.$file_name;
                                            //dd('WITH FILS');
         
                                    }
                                }
                                    
                            }else if(in_array($key, static::excluded_keys))
                            {
                                //$model[$key] = $value == 'on' ? true : false;

                            }else if(in_array($key, static::array_keys))
                            {   
                               
                                $model[$key] = json_encode($value);
                            }
                              

                            
                        }

                        //end of loop add the cover photo
                        if(isset($cover_photo_paths[$i_index]))
                        {

                             $is_save = false;
                             if(Request::hasFile('cover_photo_path'))
                             {
                                $key = 'cover_photo_path';
                                $files = Request::file($key);
                                foreach($files as $file_index => $file)
                                {
                                    if($file_index == $i_index)
                                    {
                                        $destination    =   SETTING::getPhotoPath('GeneralContent_cover_photo');
                                            $i = 1;
                                            $img = Image::make($file->getRealPath());
                                            $extension      =   'jpeg'; //$file->getClientOriginalExtension();
                                            $folder = public_path().'/'.$destination;
                                            //$file_name = $file->getClientOriginalName().'-'.date('d-m-Y-H-i-s').'-'.$i.'.'.$extension;
                                            $file_name = 'GeneralContent_'.date('d_m_Y_H_i_s').'.'.$extension;
                                            //dd($file_name);
                                            $file->move($folder,$file_name);
                                            //dd($destination);
                                            //$photo_path  = Image::make($new_file)->resize(200, 200)->save();
                                            //save into directory and update path to database

                                            if($model[$key]){
                                                @unlink(public_path($model[$key]));
                                            }
                                            //dd($model);
                                            $model[$key] = $destination.$file_name;
                                            $is_save = true;

                                    }
                                }
                                if($is_save == false)
                                {
                                    if($is_with_attachment == false)
                                    {
                                         $model['cover_photo_path'] = null;
                                    }
                                   
                                }

                             }else{

                                if($is_with_attachment == false)
                                    {
                                         $model['cover_photo_path'] = null;
                                    }
                             }


                        }else{
                            if($is_with_attachment == false)
                            {
                                 $model['cover_photo_path'] = null;
                            }
                        }


                            
                        
                        foreach($default_keys as $d_key => $d_value){
                             $this->$d_key = $d_value;
                        }
                        
                        if (!$model['id']) {
                            $model['created_by']       =   Auth::id() ? Auth::id():0;
                            $model['updated_by']       =   0;
                            $model['leaf_group_id']    =   Company::get_group_id();
                            $model['updated_by']       =   Auth::id() ? Auth::id():0;
                        } else {
                            $model['updated_by']       =   Auth::id() ? Auth::id():0;
                        }
                       
                        foreach($model->attributes as $key => $value){
                            echo 'Attribute :'.$key.'<br>';
                            if(strpos($key,'_id') !== false)
                            {
                                if($key == 'id'){

                                  continue;
                                }
                                if($value == '' || $value == null){

                                    $model[$key] = 0;
                                }
                            }
                        }
                        //dd($model);
                        $model['sort_order'] = $model['sort_order'] == '' ? 0 : $model['sort_order'];
                        $model['rating'] = isset($model['rating']) ? ($model['rating'] == '' ? 5 : $model['rating']) : 5 ;



                        $model->save();
            }
            
                        
        } catch (Exception $e) {
            dd($e);
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }
}
