<?php

namespace App\Models;

use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;

use DB;
use Auth;
use Schema;
use Validator;
use App\Company;
use App\PaymentTestingAllowList;
use App\Setting;
use App\UserAssign;
use App\PowerMeterModel\CustomerPowerUsageSummary;


class User extends Authenticatable
{
    private $extendModel;
    protected $table = 'users';
    public $timestamps = true;
    protected $listing_except_columns = ['leaf_id_user','store_id','leaf_id_group','remember_token','created_by','updated_by','created_at','updated_at','is_admin','module_access','power_meter_start_charging_date','is_super_admin','id_house_member','power_mangement_start_charging_date','customer_power_usage_summary_id','language_id','photo','month_usages'];

    protected $guarded = [];
    const default_user_photo = 'version_2\images\icons\robot_pet.png';

    use Notifiable;

    public function __construct()
    {
        $this->extendModel = new ExtendModel();
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of Accessors & Mutators
    |--------------------------------------------------------------------------
    |
    */

    public function getUserGroupIdAttribute($value)
    {
        return UserAssign::where('id','=',$this->id)
                            ->where('leaf_group_id','=',Company::get_group_id())
                            ->pluck('user_group_id');
    }


    const remove_cpus_model_mappers = ['customer_id' => 0 , 'customer_name' => '-' , 'leaf_id_user' => 0 , 'id_house_member' => 0 ];
    public function removeCurrentRoomFromCustomerSummaryData()
    {
      
         $cpus_model = CustomerPowerUsageSummary::find($this->getCustomerPowerUsageSummaryId());
        if(isset($cpus_model['id']))
        {
            foreach (static::remove_cpus_model_mappers as $key => $value)
            {
                $cpus_model[$key] = $value;
            }

            $cpus_model->save();    
        }
        
        $leaf_group_id = Company::get_group_id() == 0  ? 0 : Company::get_group_id();
        $temp = (array) json_decode($this->customer_power_usage_summary_id);

        $is_exist = isset($temp[$leaf_group_id]) ? 1: 0;
        if($is_exist)
        {
            unset($temp[$leaf_group_id]);
        }

        $this->customer_power_usage_summary_id = json_encode($temp);
        $this->save();

    }


    public static function getUserByLeafIdUser($leaf_id_user)
    {
        return User::where('leaf_id_user','=',$leaf_id_user)
                    ->first();
    }

    public static function get_model_by_leaf_id_user($leaf_id_user)
    {  
        return static::where('leaf_id_user','=',$leaf_id_user)
                ->first();
    }

    public static function get_profile_pic_by_id($user_id)
    {
        $model = User::where('id','=',$user_id)
                    ->first();

        return $model['photo'] != '' ? $model['photo'] : static::default_user_photo ;
    }

    public static function get_user_by_email($email)
    {
        $model = User::where('email','=',$email)
                    ->first();

        return $model;
    }


    /*
    |--------------------------------------------------------------------------
    | Here to manage of relationships
    |--------------------------------------------------------------------------
    |
    */

    public function creator()
    {
        return $this->belongsTo('App\User', 'created_by');
    }

    public function updator()
    {
        return $this->belongsTo('App\User', 'updated_by');
    }

    public function store()
    {
        return $this->belongsTo('App\Store', 'store_id');
    }

    public function user_group()
    {
        return $this->belongsTo('App\Store', 'store_id');
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of scope
    |--------------------------------------------------------------------------
    |
    */

    protected static function boot()
    {
       /* parent::boot();

        static::addGlobalScope('owned_by', function (Builder $builder) {
            $builder->leftJoin('user_assigns','user_assigns.id','=','users.assign_id')
                        ->where('leaf_group_id', '=', Company::get_group_id());
        });*/
    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of index listing displayed
    |--------------------------------------------------------------------------
    |
    */

    public function table_cols()
    {
        $except = $this->listing_except_columns;

        return array_diff(Schema::getColumnListing($this->table), $except);
    }

    public function listing_header()
    {
        return array_diff($this->table_cols(), $this->listing_except_columns);
    }

    public function scopeListing($query) 
    {
        $listing = $this->table_cols();
        $listing[0] = 'id';
        return $query->select(array_diff($listing, $this->listing_except_columns));
        // return $query->select(['photo','fullname','email','status','created_by','updated_by','user_id']);
    }

    public function display_relationed($parent, $col)
    {
        $parent = str_replace('_id', '', $parent);
        return ($this->$parent ? $this->$parent->$col:'');
    }

    public function display_status_string($col)
    {
        return ($this->$col ? Language::trans('Enabled'):Language::trans('Disabled'));
    }

    public function profile_jpg()
    {
        //dd(Auth::id());
        $user = static::find(Auth::id());
        if (isset($user['user_photo'])) {
            if($user['user_photo'] != ''){
                 return Setting::photo_place($user['user_photo']);
            }  
        }

        return static::default_user_photo;


    }

    /*
    |--------------------------------------------------------------------------
    | Here to manage of validation & save form
    |--------------------------------------------------------------------------
    |
    */
    public function create_user_by_leaf_id_user($leaf_member_detail)
    {

    }

    public function get_or_create_user_account($data)
    {
        $data['leaf_id_user'] = isset($data['leaf_id_user']) ? $data['leaf_id_user'] : (isset($data['id_user']) ? $data['id_user'] : 0);
        if (!$model = static::where('leaf_id_user','=',$data['leaf_id_user'])->first()) {
            $model = new static();
            $input['photo']         =   isset($data['user_photo']) ?  $data['user_photo'] : '';
            $input['fullname']      =   isset($data['user_fullname']) ? $data['user_fullname'] : '' ;
            $input['email']         =   isset($data['user_email']) ?  $data['user_email'] : '';
            $input['status']        =   true;
            $input['leaf_id_user']  =   isset($data['leaf_id_user']) ?  $data['leaf_id_user'] : '';
            //company setting
            $user_group_id          =   isset($data['user_group_id']) ? $data['user_group_id']:0;

            $model->save_form($input);
            $model->create_group($model->id, $user_group_id);
        }
        return $model;
    }

    public function remove_group($id)
    {
        UserAssign::where('id','=',$user_id)->where('leaf_group_id','=',Company::get_group_id())->delete();
    }

    public function create_group($user_id, $group_id)
    {
        DB::beginTransaction();
        try {

            $model = UserAssign::get_model_by_user_and_user_group_id($user_id,$group_id) !== null  ? UserAssign::get_model_by_user_and_user_group_id($user_id,$group_id) : new UserAssign();
            $model->id         =   $user_id;
            $model->leaf_group_id   =   Company::get_group_id();
            $model->user_group_id   =   $group_id;
            $model->save();
        } catch (Exception $e) {
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }

    public function validate_form($input)
    {
        $rules = [];

        if (!$this->id) {
            $rules['email'] =   'required';
        }

        $validator = Validator::make($input, $rules);
        if ($validator->fails()) {
            return $validator;
        }
        return false;
    }

    public static function combobox_email_vs_email()
    {
        return static::orderBy('email','asc')
                        ->pluck('email','email');
    }


    public static function repositionArrayElement(array &$array, $key, int $order): void
    {
        if(($a = array_search($key, array_keys($array))) === false){
            throw new \Exception("The {$key} cannot be found in the given array.");
        }
        $p1 = array_splice($array, $a, 1);
        $p2 = array_splice($array, 0, $order);
        $array = array_merge($p2, $p1, $array);
    }

    public static function user_combobox($user_id=0)
    {
        $return =  static::orderBy('fullname','asc');
        if($user_id !=0)
        {
             $return = $return->where('id','!=',$user_id);
        } 

        $return = $return->pluck('fullname','id');

        if($user_id !=0)
        {
            $return_2 = static::orderBy('fullname','asc')
                                ->where('id','=',$user_id)
                                ->pluck('fullname','id');

            $return = array_merge($return->toArray(),$return_2->toArray());
            //dd($return);
           // User::repositionArrayElement($return, count($return)  -1 , 0 );
        }
        
        return  $return;
    }

    public static function user_assign_combobox($user_assign_id=0)
    {
        $assigned_user_id = array_column(UserAssign::getByUserGroupIdAndNotUserAssignId(Company::get_group_id(),$user_assign_id)->toArray(),'user_id');
        //dd($assigned_user_id);
        $return = static::orderBy('fullname','asc');
        if(count($assigned_user_id) > 0)
        {
             $return = $return->whereNotIn('id',$assigned_user_id);
        }
       

        $return = $return->pluck('fullname','id');


        return  $return;
    }


    public static function combobox()
    {
        $return = static::all()
                    ->pluck('id','name');

        return $return;
    }

    public function get_user_by_range($from, $to, $col)
    {
        $leaf_api = new LeafAPI();
        $listing = $leaf_api->get_member_list();
        $datas = [];
        $insert=false;
        // process if leaf api return status code = 1
        if ($listing['status_code'] == 1) {
            // get users listing in leaf api
            foreach ($listing['member'] as $row) {
                if ($row['user_name'] == $from) {
                    $insert=true;
                }
                if ($insert) {
                    $datas[] = $row[$col];
                }
                if ($row['user_name'] == $to) {
                    $insert=false;
                }
            }
        }
        return $datas;
    }

    public function save_form($input)
    {
        DB::beginTransaction();
        try {
            foreach ($input as $key => $value) {
                if ($key != '_token' && $key != 'status_code' && $key != 'user_group_id' && $key != 'power_mangement_start_charging_date') {
                    $this->$key = (string) $value;
                }else if($key == 'power_mangement_start_charging_date'){
                    $this->$key = date('Y-m-d H:m' , strtotime($value));
                }
            }
            if (!$this->id) {
                $this->created_by = Auth::id() ? Auth::id():0;
                $this->updated_by = 0;
            } else {
                $this->updated_by = Auth::id() ? Auth::id():0;
            }

            $this->save();

        } catch (Exception $e) {
            throw $e;
            DB::rollBack();
        }
        DB::commit();
    }
}

